/**
 * 对公众平台发送给公众账号的消息加解密示例代码.
 * 
 * @copyright Copyright (freedbconnlist) 1998-2014 Tencent Inc.
 */

// ------------------------------------------------------------------------

/**
 * 针对org.apache.commons.codec.binary.Base64?
 * ?导入架包commons-codec-1.9（或commons-codec-1.8等其他版本）
 * 官方下载地址：http://commons.apache.org/proper/commons-codec/download_codec.cgi
 */
package com.cnd3b.utility.qywechat.base.system;

import org.apache.commons.codec.binary.Base64;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Random;

/**
 * 提供接收和推送给公众平台消息的加解密接口(UTF8编码的字符串).
 * <ol>
 * <li>第三方回复加密消息给公众平台</li>
 * <li>第三方收到公众平台发送的消息，验证消息的安全性，并对消息进行解密?/li>
 * </ol>
 * 说明：异常java.security.InvalidKeyException:illegal Key Size的解决方?
 * <ol>
 * <li>在官方网站下载JCE无限制权限策略文件（JDK7的下载地?
 * http://www.oracle.com/technetwork
 * /java/javase/downloads/jce-7-download-432124.html</li>
 * <li>下载后解压，可以看到local_policy.jar和US_export_policy.jar以及readme.txt</li>
 * <li>如果安装了JRE，将两个jar文件放到%JRE_HOME%\lib\security目录下覆盖原来的文件</li>
 * <li>如果安装了JDK，将两个jar文件放到%JDK_HOME%\jre\lib\security目录下覆盖原来文?/li>
 * </ol>
 */
public class WXBizMsgCrypt {
	static Charset CHARSET = StandardCharsets.UTF_8;
	Base64 base64 = new Base64();
	byte[] aesKey;
	String token;
	String appId;

	/**
	 * 构?函数
	 * 
	 * @param token
	 *            公众平台上，?者设置的token
	 * @param encodingAesKey
	 *            公众平台上，?者设置的EncodingAESKey
	 * @param appId
	 *            公众平台appid
	 * 
	 * @throws AesException
	 *             执行失败，请查看该异常的错误码和具体的错误信?
	 */
	public WXBizMsgCrypt(String token, String encodingAesKey, String appId)
			throws com.cnd3b.utility.qywechat.base.system.AesException {
		if (encodingAesKey.length() != 43) {
			throw new AesException(com.cnd3b.utility.qywechat.base.system.AesException.IllegalAesKey);
		}

		this.token = token;
		this.appId = appId;
		aesKey = Base64.decodeBase64(encodingAesKey + "=");
	}

	// 生成4个字节的网络字节?
	byte[] getNetworkBytesOrder(int sourceNumber) {
		byte[] orderBytes = new byte[4];
		orderBytes[3] = (byte) (sourceNumber & 0xFF);
		orderBytes[2] = (byte) (sourceNumber >> 8 & 0xFF);
		orderBytes[1] = (byte) (sourceNumber >> 16 & 0xFF);
		orderBytes[0] = (byte) (sourceNumber >> 24 & 0xFF);
		return orderBytes;
	}

	// 还原4个字节的网络字节?
	int recoverNetworkBytesOrder(byte[] orderBytes) {
		int sourceNumber = 0;
		for (int i = 0; i < 4; i++) {
			sourceNumber <<= 8;
			sourceNumber |= orderBytes[i] & 0xff;
		}
		return sourceNumber;
	}

	// 随机生成16位字符串
	String getRandomStr() {
		String base = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
		Random random = new Random();
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < 16; i++) {
			int number = random.nextInt(base.length());
			sb.append(base.charAt(number));
		}
		return sb.toString();
	}

	/**
	 * 对明文进行加?
	 * 
	 * @param text
	 *            ?加密的明?
	 * @return 加密后base64编码的字符串
	 * @throws AesException
	 *             aes加密失败
	 */
	String encrypt(String randomStr, String text) throws com.cnd3b.utility.qywechat.base.system.AesException {
		com.cnd3b.utility.qywechat.base.system.ByteGroup byteCollector = new com.cnd3b.utility.qywechat.base.system.ByteGroup();
		byte[] randomStrBytes = randomStr.getBytes(CHARSET);
		byte[] textBytes = text.getBytes(CHARSET);
		byte[] networkBytesOrder = getNetworkBytesOrder(textBytes.length);
		byte[] appidBytes = appId.getBytes(CHARSET);

		// randomStr + networkBytesOrder + text + appid
		byteCollector.addBytes(randomStrBytes);
		byteCollector.addBytes(networkBytesOrder);
		byteCollector.addBytes(textBytes);
		byteCollector.addBytes(appidBytes);

		// ... + pad: 使用自定义的填充方式对明文进行补位填?
		byte[] padBytes = com.cnd3b.utility.qywechat.base.system.PKCS7Encoder.encode(byteCollector.size());
		byteCollector.addBytes(padBytes);

		// 获得?的字节流, 未加?
		byte[] unencrypted = byteCollector.toBytes();

		try {
			// 设置加密模式为AES的CBC模式
			Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
			SecretKeySpec keySpec = new SecretKeySpec(aesKey, "AES");
			IvParameterSpec iv = new IvParameterSpec(aesKey, 0, 16);
			cipher.init(Cipher.ENCRYPT_MODE, keySpec, iv);

			// 加密
			byte[] encrypted = cipher.doFinal(unencrypted);

			// 使用BASE64对加密后的字符串进行编码
			return base64.encodeToString(encrypted);
		} catch (Exception e) {
			e.printStackTrace();
			throw new com.cnd3b.utility.qywechat.base.system.AesException(com.cnd3b.utility.qywechat.base.system.AesException.EncryptAESError);
		}
	}

	/**
	 * Ľн.
	 * 
	 * @param text
	 *            Ҫܵ
	 * @return ܵõ
	 * @throws AesException
	 *             aesʧ
	 */
	String decrypt(String text) throws com.cnd3b.utility.qywechat.base.system.AesException {
		byte[] original;
		try {
			// 设置解密模式为AES的CBC模式
			Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
			SecretKeySpec key_spec = new SecretKeySpec(aesKey, "AES");
			IvParameterSpec iv = new IvParameterSpec(Arrays.copyOfRange(aesKey,
					0, 16));
			cipher.init(Cipher.DECRYPT_MODE, key_spec, iv);
			// 使用BASE64对密文进行解?
			byte[] encrypted = Base64.decodeBase64(text);
			// 解密
			original = cipher.doFinal(encrypted);
		} catch (Exception e) {
			e.printStackTrace();
			throw new com.cnd3b.utility.qywechat.base.system.AesException(com.cnd3b.utility.qywechat.base.system.AesException.DecryptAESError);
		}

		String xmlContent, from_appid;
		try {
			// 去除补位字符
			byte[] bytes = com.cnd3b.utility.qywechat.base.system.PKCS7Encoder.decode(original);

			// 分离16位随机字符串,网络字节序和AppId
			byte[] networkOrder = Arrays.copyOfRange(bytes, 16, 20);

			int xmlLength = recoverNetworkBytesOrder(networkOrder);

			xmlContent = new String(Arrays.copyOfRange(bytes, 20,
					20 + xmlLength), CHARSET);
			from_appid = new String(Arrays.copyOfRange(bytes, 20 + xmlLength,
					bytes.length), CHARSET);
		} catch (Exception e) {
			e.printStackTrace();
			throw new com.cnd3b.utility.qywechat.base.system.AesException(com.cnd3b.utility.qywechat.base.system.AesException.IllegalBuffer);
		}

		// appid不相同的情况
		if (!from_appid.equals(appId)) {
			throw new com.cnd3b.utility.qywechat.base.system.AesException(com.cnd3b.utility.qywechat.base.system.AesException.ValidateAppidError);
		}
		return xmlContent;

	}

	/**
	 * 将公众平台回复用户的消息加密打包.
	 * <ol>
	 * <li>对要发?的消息进行AES-CBC加密</li>
	 * <li>生成安全签名</li>
	 * <li>将消息密文和安全签名打包成xml格式</li>
	 * </ol>
	 * 
	 * @param replyMsg
	 *            公众平台待回复用户的消息，xml格式的字符串
	 * @param timeStamp
	 *            时间戳，可以自己生成，也可以用URL参数的timestamp
	 * @param nonce
	 *            随机串，可以自己生成，也可以用URL参数的nonce
	 * 
	 * @return 加密后的可以直接回复用户的密文，包括msg_signature, timestamp, nonce,
	 *         encrypt的xml格式的字符串
	 * @throws AesException
	 *             执行失败，请查看该异常的错误码和具体的错误信?
	 */
	public String encryptMsg(String replyMsg, String timeStamp, String nonce)
			throws com.cnd3b.utility.qywechat.base.system.AesException {
		// 加密
		String encrypt = encrypt(getRandomStr(), replyMsg);

		// 生成安全签名
		if (timeStamp.equals("")) {
			timeStamp = Long.toString(System.currentTimeMillis());
		}

		String signature = com.cnd3b.utility.qywechat.base.system.SHA1.getSHA1(token, timeStamp, nonce, encrypt);


		// 生成发?的xml
		return com.cnd3b.utility.qywechat.base.system.XMLParse.generate(encrypt, signature, timeStamp, nonce);
	}

	/**
	 * ?消息的真实?，并且获取解密后的明?
	 * <ol>
	 * <li>利用收到的密文生成安全签名，进行签名验证</li>
	 * <li>若验证?过，则提取xml中的加密消息</li>
	 * <li>对消息进行解?/li>
	 * </ol>
	 * 
	 * @param msgSignature
	 *            签名串，对应URL参数的msg_signature
	 * @param timeStamp
	 *            时间戳，对应URL参数的timestamp
	 * @param nonce
	 *            随机串，对应URL参数的nonce
	 * @param postData
	 *            密文，对应POST请求的数?
	 * 
	 * @return 解密后的原文
	 * @throws AesException
	 *             执行失败，请查看该异常的错误码和具体的错误信?
	 */
	public String decryptMsg(String msgSignature, String timeStamp,
			String nonce, String postData) throws com.cnd3b.utility.qywechat.base.system.AesException {

		// 密钥，公众账号的app secret
		// 提取密文
		Object[] encrypt = XMLParse.extract(postData);
		// 验证安全签名
		String signature = com.cnd3b.utility.qywechat.base.system.SHA1.getSHA1(token, timeStamp, nonce,
				encrypt[1].toString());
		// 和URL中的签名比较是否相等
		if (!signature.equals(msgSignature)) {
			throw new com.cnd3b.utility.qywechat.base.system.AesException(com.cnd3b.utility.qywechat.base.system.AesException.ValidateSignatureError);
		}
		// 解密
		return decrypt(encrypt[1].toString());
	}

	/**
	 * 验证URL
	 * 
	 * @param msgSignature
	 *            签名串，对应URL参数的msg_signature
	 * @param timeStamp
	 *            时间戳，对应URL参数的timestamp
	 * @param nonce
	 *            随机串，对应URL参数的nonce
	 * @param echoStr
	 *            随机串，对应URL参数的echostr
	 * 
	 * @return 解密之后的echostr
	 * @throws AesException
	 *             执行失败，请查看该异常的错误码和具体的错误信?
	 */
	public String verifyUrl(String msgSignature, String timeStamp,
			String nonce, String echoStr) throws com.cnd3b.utility.qywechat.base.system.AesException {
		String signature = com.cnd3b.utility.qywechat.base.system.SHA1.getSHA1(token, timeStamp, nonce, echoStr);
		if (!signature.equals(msgSignature)) {
			throw new com.cnd3b.utility.qywechat.base.system.AesException(com.cnd3b.utility.qywechat.base.system.AesException.ValidateSignatureError);
		}
		return decrypt(echoStr);
	}

}